//////////////////////////////////////////////////////////////////////////////////
// concorGM                                                                               
//                                                                               
//                  For each pair (Xi,Yj), valuation of the partial link
//                          between subset Xi and subset Yj 
//                        
//                               SUCCESSIVE SOLUTIONS                             
//                                                                               
//////////////////////////////////////////////////////////////////////////////////

function [u,v,criGM]=concorGM(X,Y,px,py,r)

//////////////////////////////////////////////////////////////////////////////////
// X and Y are 2 data matrices, n x p and n x q, of p variables and q variables
// (centered).
// The row vector px contains the kx numbers of variables of the kx subsets of X.
// The row vector py contains the ky numbers of variables of the ky subsets of Y.
// sum(px)=p and sum(py)=q.
// When kx =1 (px=p), use concor.m              
// r is the wanted number of solutions (< 1+min(rank(Xi'Yj))), and thus often 
// rmax = min(min(px),min(py),n). 
// The blocks ui of u (p x r) are the orthonormed partial axes of Xi.
// The blocks vj of v (q x r) are the orthonormed partial axes of Yj.
// criGM (kx x ky x r) contains the link cov2(Xiui,Yjvj) of each pair (Xi,Yj).
//
// For each solution, sumi sumj cov2(Xiui,Yjvj) is the optimized criterion,
// partial components: Xiui and Yjvj,
// mean component associated to Xiui: sumj cov(Xiui,Yjvj)*Yjvj.
// 
// px+py orthogonality constraints for a new solution : ui'*ui=Ir and vj'*vj=Ir.
//
// 
// Kissita, Cazes, Hanafi & Lafosse (2004) in Revue de Statistique Appliquee.
// hanafi@enitiaa-nantes.fr
// Roger.Lafosse@lsp.ups-tlse.fr
// cazes@ceremade.dauphine.fr
// gakissita@yahoo.fr
/////////////////////////////////////////////////////////////////////////////////

[u,v,cri]=svdbip(X'*Y,px,py,r);
criGM=cri./size(X,1)^2;	

endfunction
