////////////////////////////////////////////////////////////////////////////////
// concorGMCANO                                                                
//  
//                      Canonical analysis of subsets Yj
//  
//                            with subsets Xi.                                       
//
//              Relative valuations by squared correlations
//           of the proximities of subsets Xi with subsets Yj 
//                        
//                                                  SUCCESSIVE SOLUTIONS       
//
//concorCANO generalized to 2 sets of subsets.                      
///////////////////////////////////////////////////////////////////////////////

function [Px,rx,u,Py,ry,v,rho2]=concorGMCANO(X,Y,px,py,r)

///////////////////////////////////////////////////////////////////////////////
// X and Y are 2 data matrices, n x p and n x q, of p variables and q variables
// (centered).
// The row vector px contains the kx numbers of variables of the kx subsets of X.
// The row vector py contains the ky numbers of variables of the ky subsets of Y.
// sum(px)=p and sum(py)=q.
// When kx =1 (px=p), use concorCANO.m              
// r is the wanted number of solutions (< 1+min(rank(Xi'Yj))).
// 
// The kx blocks ui (rx(i) x r) of u, are formed with r orthonormed vectors.
// P1=P(:,1:rx(1)), P2=P(:,rx(1)+1 : rx(1)+rx(2)), P3 =....
// Each matrix Xi has been standardized for obtaining the matrix Pi and the
// standardized canonical components Piui. 
// P1=P(:,1:rx(1)), P2=P(:,rx(1)+1 : rx(1)+rx(2)), P3 =....
// P1u1=P(:,1:rx(1))*u(1:rx(1),:).
// Idem for Py, ry , and v.
//
// For each pair (Xi,Yj), and each solution k, rho2 contains the squared
// correlations rho2ijk=rho2(Piui(:,k),Yjvj(:,k)). 
//
// For each solution k, sumi sumj rho2ijk is the optimized criterion,
// 
// px+py orthogonality constraints for a new solution : ui'*ui=Ir and vj'*vj=Ir.
// 
// Kissita G. Thesis Paris 9 (2003) 
// gakissita@yahoo.fr
///////////////////////////////////////////////////////////////////////////// 
// rx and ry contains maximal values. These values may be excessive.
n=size(X,1);
rx=zeros(1,length(px));Px=[];
cx=[0,cumsum(px)];
for j=1:length(px)
rx(j)=rank(X(:,cx(j)+1:cx(j+1)));
[pp,d,qq]=svd(X(:,cx(j)+1:cx(j+1)));
Px=[Px,pp(:,1:rx(j))*sqrt(n)];
end

ry=zeros(1,length(py));Py=[];
cy=[0,cumsum(py)];
for j=1:length(py)
ry(j)=rank(Y(:,cy(j)+1:cy(j+1)));
[pp,d,qq]=svd(Y(:,cy(j)+1:cy(j+1)));
Py=[Py,pp(:,1:ry(j))*sqrt(n)];
end
 if r > min([min(rx),min(ry),n])
    display(' r IS TOO HIGH')
    return
 else
 end


[u,v,cri]=svdbip(Px'*Py,rx,ry,r);
rho2=cri./n^2;	


endfunction
