////////////////////////////////////////////////////////////////////////////////
//  concorGMREG                                                                //
//  
//                      Regression of subsets Yj
//  
//                           by subsets Xi.                                       
//
//              Relative valuations by explained variances
//         between one explanatory subset Xi and one explained subset Yj 
//                        
//                                                  SUCCESSIVE SOLUTIONS       //
//
//concorREG generalized to several explanatory matrices                        //
///////////////////////////////////////////////////////////////////////////////

function [P,rx,u,v,varexp]=concorGMREG(X,Y,px,py,r)

///////////////////////////////////////////////////////////////////////////////
// X and Y are 2 data matrices, n x p and n x q, of p variables and q variables
// (centered).
// The row vector px contains the kx numbers of variables of the kx subsets of X.
// The row vector py contains the ky numbers of variables of the ky subsets of Y.
// sum(px)=p and sum(py)=q.
// When kx =1 (px=p), use concor.m              
// r is the wanted number of solutions (< 1+min(rank(Xi'Yj))), and thus often 
// rmax = min(min(px),min(py),n).
// 
// The ky blocks vj (pyj x r) of v (q x r) are the orthonormed partial axes of Yj.
// varexp (kx x ky x r) contains the explained variances.
// The kx blocks ui (rx(i) x r) of u, are formed with r orthonormed vectors.
// P1=P(:,1:rx(1)), P2=P(:,rx(1)+1 : rx(1)+rx(2)), P3 =....
// Each matrix Xi has been standardized for obtaining the matrix Pi and the
// standardized partial explanatory components Piui. 
// P1=P(:,1:rx(1)), P2=P(:,rx(1)+1 : rx(1)+rx(2)), P3 =....
// P1u1=P(:,1:rx(1))*u(1:rx(1),:).
// For each pair (Xi,Yj, and each solution k, varexp contains the explained
// variances varexpijk = rho2(Piui(:,k),Yjvj(:,k)) varYjvj(:,k). 
//
// For each solution k, sumi sumj varexpijk is the optimized criterion,
// 
// px+py orthogonality constraints for a new solution : ui'*ui=Ir and vj'*vj=Ir.
// 
// Hanafi & Lafosse (2004) in COMPSTAT 2004.
// hanafi@enitiaa-nantes.fr
// Roger.Lafosse@lsp.ups-tlse.fr
/////////////////////////////////////////////////////////////////////////////////
// rk here are maximal values. Smaller values may be better in some cases. 

rx=zeros(1,length(px));P=[];
n=size(X,1);
cx=[0,cumsum(px)];
for j=1:length(px)
rx(j)=rank(X(:,cx(j)+1:cx(j+1)));
[pp,d,qq]=svd(X(:,cx(j)+1:cx(j+1)));
P=[P,pp(:,1:rx(j))*sqrt(n)];
end

[u,v,cri]=svdbip(P'*Y,rx,py,r);
varexp=cri./size(X,1)^2;	

endfunction
