//////////////////////////////////////////////////////////////////////////////////////
// As svdbip, but with another (less fast) algorithm                                //
//                                                                                  //
//                          SVD for bipartitioned matrix                           
//                                                                                  //
//                             Successive Solutions
//                                                                       
//////////////////////////////////////////////////////////////////////////////////////

function [u,v,criterion]=svdbip2(x,K,H,r)

//////////////////////////////////////////////////////////////////////////////////////
//  SVD for a matrix x partitioned by rows (K) and by columns (H).
// Contrarly to svdbip, this function is based on an algorithm without svd.
// So, divisions for norming vectors might be a problem with svdbip2.
// Contrary to svdbip, the stopping rule here is built with the vectors and not with
// the optimized criterion. Then the calculus is less fast also for this reason.
//
// Kissita Thesis Ceremade Paris Dauphine (2003)
//
//////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////
// Example
//x=rand(10,20);
//[u1,v1,cri1]=svdbip(x,[4,6],[3,7,10],3);
//[u2,v2,cri2]=svdbip2(x,[4,6],[3,7,10],3);
//cri2,cri1
//////////////////////////////////////////////////////////////////////////////////////

//
// INITIALISATIONS 
[p,q]=size(x);

 if (sum(H) ~= q | sum(K) ~= p)
    display('K or H IS NOT SUITABLE')
    return
 else
 end

M=size(K,2);
N=size(H,2);
u=zeros(p,r);u1=u;
v=zeros(q,r);v1=v;
ck=cumsum([0,K]);
ch=cumsum([0,H]);
criterion=zeros(M,N,r);

// ALGORITHM
for k=1:r
 a=2;
 //comp=0;

// PROPOSED INITIALISATION OF THE ALGORITHM, for u and v
 for i=1:M
   for j=1:N
    ak=ck(i)+1:ck(i+1);
    ah=ch(j)+1:ch(j+1);
    [pp,d,qq]=svd(x(ak,ah));
    u(ak,k)=pp(:,1);v(ah,k)=qq(:,1);
   end
 end

 while (a > %eps)

  u1=u(:,k);v1=v(:,k);
  //comp=comp+1;

    v2=zeros(q,1);
   for j=1:N
     ah=ch(j)+1:ch(j+1);
      for i=1:M
       ak=ck(i)+1:ck(i+1);
       v2(ah)=v2(ah)+ x(ak,ah)'*u(ak,k)*u(ak,k)'*x(ak,ah)*v(ah,k);
      end
     a2=sqrt(v2(ah)'*v2(ah));
      //if a2 > eps
      v(ah,k)=v2(ah)/a2;
      //else
      //v(ah,k)=v2(ah);
      //end
   end

    u2=zeros(p,1);
   for i=1:M
     ak=ck(i)+1:ck(i+1);
       for j=1:N
        ah=ch(j)+1:ch(j+1);
        u2(ak)=u2(ak)+ x(ak,ah)*v(ah,k)*v(ah,k)'*x(ak,ah)'*u(ak,k);
       end
     a2=sqrt(u2(ak)'*u2(ak));
       //if a2 > eps
       u(ak,k)=u2(ak)/a2;
       //else
       //u(ak,k)=u2(ak);
       //end
   end

   a=(norm(u(:,k)-u1)/p+norm(v(:,k)-v1))/q;
 end
 
   for i=1:M
    ak=ck(i)+1:ck(i+1);
     for j=1:N
     ah=ch(j)+1:ch(j+1);
     c=u(ak,k)'*x(ak,ah)*v(ah,k);    
     x(ak,ah)=x(ak,ah)-u(ak,k)*u(ak,k)'*x(ak,ah)-x(ak,ah)*v(ah,k)*v(ah,k)'+u(ak,k)*c*v(ah,k)';   
     criterion(i,j,k)=c^2;
     end
   end
//comp

end

endfunction
 

